@@ -8,6 +8,7 @@
 #  rights to use.
 ####################################################################
 
+from __future__ import print_function
 import timeit, numpy
 
 array_size = 1e6
@@ -22,29 +23,29 @@ dtype = 'float32'
 #dtype = 'float64'
 
 def compare_times(setup, expr):
-    print "Expression:", expr
+    print("Expression:", expr)
     namespace = {}
-    exec setup in namespace
+    exec(setup, namespace)
 
     numpy_timer = timeit.Timer(expr, setup)
     numpy_time = numpy_timer.timeit(number=iterations)
-    print 'numpy:', numpy_time / iterations
+    print('numpy:', numpy_time / iterations)
 
     try:
         weave_timer = timeit.Timer('blitz("result=%s")' % expr, setup)
         weave_time = weave_timer.timeit(number=iterations)
-        print "Weave:", weave_time/iterations
+        print("Weave:", weave_time/iterations)
 
-        print "Speed-up of weave over numpy:", round(numpy_time/weave_time, 2)
+        print("Speed-up of weave over numpy:", round(numpy_time/weave_time, 2))
     except:
-        print "Skipping weave timing"
+        print("Skipping weave timing")
 
     numexpr_timer = timeit.Timer('evaluate("%s", optimization="aggressive")' % expr, setup)
     numexpr_time = numexpr_timer.timeit(number=iterations)
-    print "numexpr:", numexpr_time/iterations
+    print("numexpr:", numexpr_time/iterations)
 
     tratio = numpy_time/numexpr_time
-    print "Speed-up of numexpr over numpy:", round(tratio, 2)
+    print("Speed-up of numexpr over numpy:", round(tratio, 2))
     return tratio
 
 setup1 = """\
@@ -132,14 +133,14 @@ def compare(check_only=False):
         total += compare_times(*params)
         print
     average = total / len(experiments)
-    print "Average =", round(average, 2)
+    print("Average =", round(average, 2))
     return average
 
 if __name__ == '__main__':
     import numexpr
-    numexpr.print_versions()
+    print("Numexpr version: ", numexpr.__version__)
 
     averages = []
     for i in range(iterations):
         averages.append(compare())
-    print "Averages:", ', '.join("%.2f" % x for x in averages)
+    print("Averages:", ', '.join("%.2f" % x for x in averages))
